'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _qs = require('qs');

var _qs2 = _interopRequireDefault(_qs);

var _humps = require('humps');

var _api = require('../modules/api');

var apiActions = _interopRequireWildcard(_api);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function getDefaultHeaders() {
  return {
    'Accept': 'application/json',
    'Content-Type': 'application/json'
  };
}

function serialize(resource) {
  return JSON.stringify(resource);
}

function handleErrors(response) {
  if (!response.ok) {
    var error = new Error(response.statusText);
    error.status = response.status;
    throw error;
  }

  return response;
}

function handleResponse(response) {
  return response.json().then(function (_ref) {
    var data = _ref.data;
    var _ref$included = _ref.included;
    var included = _ref$included === undefined ? [] : _ref$included;
    var _ref$meta = _ref.meta;
    var meta = _ref$meta === undefined ? {} : _ref$meta;

    return {
      resources: [].concat(_toConsumableArray(Array.isArray(data) ? data : [data]), _toConsumableArray(included)),
      result: Array.isArray(data) ? data.map(function (r) {
        return r.id;
      }) : data.id,
      meta: meta
    };
  });
}

function createMiddleware(host) {
  var _requestActions;

  var defaultHeaders = arguments.length <= 1 || arguments[1] === undefined ? getDefaultHeaders() : arguments[1];

  var getURL = function getURL(resource, params) {
    var urlParts = [host];

    if (resource.type) urlParts = [].concat(_toConsumableArray(urlParts), ['/', (0, _humps.decamelize)(resource.type)]);
    if (resource.id) urlParts = [].concat(_toConsumableArray(urlParts), ['/', resource.id]);
    if (params) urlParts = [].concat(_toConsumableArray(urlParts), ['?', _qs2.default.stringify(params)]);

    return urlParts.join('');
  };

  var requestAction = function requestAction(method, _ref2) {
    var resource = _ref2.resource;
    var params = _ref2.params;
    var headers = _ref2.headers;

    var url = getURL(resource, params);

    return fetch(url, {
      method: method,
      body: method !== 'GET' ? serialize({ data: resource }) : undefined,
      headers: _extends({}, defaultHeaders, headers)
    }).then(function (response) {
      return handleErrors(response);
    }).then(function (response) {
      return handleResponse(response);
    });
  };

  var requestActions = (_requestActions = {}, _defineProperty(_requestActions, apiActions.GET, function (options) {
    return requestAction('GET', options);
  }), _defineProperty(_requestActions, apiActions.POST, function (options) {
    return requestAction('POST', options);
  }), _defineProperty(_requestActions, apiActions.PUT, function (options) {
    return requestAction('PUT', options);
  }), _defineProperty(_requestActions, apiActions.PATCH, function (options) {
    return requestAction('PATCH', options);
  }), _defineProperty(_requestActions, apiActions.DELETE, function (options) {
    return requestAction('DELETE', options);
  }), _requestActions);

  return function (store) {
    return function (next) {
      return function (action) {
        if (requestActions[action.type]) {
          next(action);

          return requestActions[action.type](action.payload).then(function (data) {
            store.dispatch(apiActions.receive(data.resources));
            return data;
          });
        }

        return next(action);
      };
    };
  };
}

exports.default = createMiddleware;